@@ -1,9 +1,11 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: GAfit.cpp,v 1.6 2005/05/13 18:54:48 wojdyr Exp $
+// $Id: GAfit.cpp,v 1.12 2006/07/10 15:58:50 wojdyr Exp $
 
 #include "common.h"
 #include "GAfit.h"
 #include "ui.h"
+#include "settings.h"
+#include "numfuncs.h"
 #include <stdlib.h>
 #include <time.h>
 #include <algorithm>
@@ -15,8 +17,8 @@
 
 using namespace std;
 
-GAfit::GAfit ()
-    :v_fit ('g', "Genetic Algorithm"),
+GAfit::GAfit()
+   : Fit("Genetic-Algorithms"),
      popsize (100), elitism(0),
      mutation_type('u'), p_mutation(0.1), mutate_all_genes(false), 
      mutation_strength(0.1), crossover_type('u'), p_crossover(0.3),
@@ -28,7 +30,7 @@ GAfit::GAfit ()
      pop(0), opop(0),
      best_indiv(0)
 {
-    default_max_iterations = 200;
+    /*
     irpar["population-size"] = IntRange (&popsize, 2, 9999);
     irpar["steady-size"] = IntRange (&elitism, 0, 9999);
     epar.insert(pair<string, Enum_string>("mutation-type", 
@@ -36,19 +38,23 @@ GAfit::GAfit ()
     fpar["p-mutation"] = &p_mutation;
     bpar["mutate-all-genes"] = &mutate_all_genes;
     fpar["mutation-strength"] = &mutation_strength;
+    */
     Crossover_enum ['u'] = "uniform";
     Crossover_enum ['o'] = "one-point";
     Crossover_enum ['t'] = "two-point";
     Crossover_enum ['a'] = "arithmetic1";
     Crossover_enum ['A'] = "arithmetic2";
     Crossover_enum ['g'] = "guaranteed-avg";
+    /*
     epar.insert(pair<string, Enum_string>("crossover-type", 
                                Enum_string (Crossover_enum, &crossover_type)));
     fpar["p-crossover"] = &p_crossover;
+    */
     Selection_enum ['r'] = "roulette";
     Selection_enum ['t'] = "tournament";
     Selection_enum ['s'] = "SRS";
     Selection_enum ['d'] = "DS";
+    /*
     epar.insert (pair<string, Enum_string>("selection-type", 
                                Enum_string (Selection_enum, &selection_type)));
     bpar["rank-scoring"] = &rank_scoring;
@@ -62,6 +68,7 @@ GAfit::GAfit ()
     fpar["wssr-stop"] = &wssr_stop;
     irpar["autoplot-indiv-nr"] = IntRange(&autoplot_indiv_nr, -1, 999999999);
     spar["GA-log-file"] = &log_file;
+    */
 }
 
 GAfit::~GAfit() {}
@@ -86,9 +93,9 @@ fp GAfit::init()
     return 0;
 }
 
-int GAfit::autoiter()
+void GAfit::autoiter()
 {
-    wssr_before = compute_wssr();
+    wssr_before = compute_wssr(a_orig, datsums);
     info ("WSSR before starting GA: " + S(wssr_before));
     assert (pop && opop); 
     if (elitism >= popsize) {
@@ -104,7 +111,6 @@ int GAfit::autoiter()
         post_selection();
     }
     post_fit (best_indiv.g, best_indiv.raw_score);
-    return 0;
 }
 
 void GAfit::log_ga_options()
@@ -115,7 +121,7 @@ void GAfit::log_ga_options()
         return;
     }
     os << "####### Current time: " << time_now() << endl;
-    istringstream iss(set_script ('f'));
+    istringstream iss(getSettings()->set_script());
     string s;
     while (getline (iss, s)) 
         os << "# " << s << endl;
@@ -125,7 +131,7 @@ void GAfit::log_ga_options()
 
 void GAfit::compute_wssr_for_ind (vector<Individual>::iterator ind)
 {
-    ind->raw_score = compute_wssr(ind->g);
+    ind->raw_score = compute_wssr(ind->g, datsums);
     ind->generation = iter_nr;
 }
 
@@ -469,8 +475,7 @@ bool GAfit::termination_criteria_and_print_info (int iter)
         generations_sum += i->generation;
     }
     fp std_dev = sq_sum > 0 ? sqrt (sq_sum / pop->size()) : 0;
-    if (iter % output_one_of == 0)
-        info ("Population #" + S(iter_nr) + ": best " + S(min) 
+    info ("Population #" + S(iter_nr) + ": best " + S(min) 
                 + ", avg " + S(avg) + ", worst " + S(tmp_max) 
                 + ", std dev. " + S(std_dev));
     if (!log_file.empty()) {
