@@ -1,45 +1,42 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: LMfit.cpp,v 1.7 2005/05/13 18:54:48 wojdyr Exp $
+// $Id: LMfit.cpp,v 1.17 2006/07/25 14:58:25 wojdyr Exp $
 
 #include "common.h"
 #include "LMfit.h"
 #include "ui.h"
+#include "settings.h"
 #include <math.h>
 #include <vector>
 #include <algorithm>
 
 using namespace std;
 
-LMfit::LMfit () 
-    : v_fit ('m', "Levenberg-Marquardt"),
-      lambda_starting_value (0.001),
-      lambda_up_factor (10), lambda_down_factor (10),
-      stop_rel (1e-4), shake_before (0), shake_type ('u'),
-      alpha(0), alpha_(0), beta(0), beta_(0)
+LMfit::LMfit() 
+    : Fit("Levenberg-Marquardt"),
+      shake_before (0), shake_type ('u')
 {
-    fpar ["lambda-starting-value"] = &lambda_starting_value;
-    fpar ["lambda-up-factor"] = &lambda_up_factor;
-    fpar ["lambda-down-factor"] = &lambda_down_factor;
-    fpar ["stop-rel-change"] = &stop_rel;
+    /*
     fpar ["shake-before"] = &shake_before;
     epar.insert(pair<string, Enum_string>("shake-type", 
                                Enum_string (Distrib_enum, &shake_type)));
+    */
 }    
     
-LMfit::~LMfit () {}
+LMfit::~LMfit() {}
 
 // WSSR is also called chi2
-fp LMfit::init ()   
+fp LMfit::init()   
 {
     alpha.resize (na*na);
     alpha_.resize (na*na);
     beta.resize (na);
     beta_.resize (na);
-    if (na < 1 ) {
-        warn ("What should I fit ?");
+    if (na < 1) {
+        warn ("No data points. What should I fit ?");
         return -1;
     }
-    lambda = lambda_starting_value;
+    lambda = getSettings()->get_f("lm-lambda-start");
+    //TODO what to do with this shake?
     if (shake_before > 0.) {
         for (int i = 0; i < na; i++) 
             a[i] = draw_a_from_distribution (i, shake_type, shake_before);
@@ -49,41 +46,37 @@ fp LMfit::init ()
 
     info (print_matrix (a, 1, na, "Initial A"));
     //no need to optimise it (and compute chi2 and derivatives together)
-    chi2 = compute_wssr (a);
-    compute_derivatives(a, alpha, beta);
+    chi2 = compute_wssr(a, datsums);
+    compute_derivatives(a, datsums, alpha, beta);
     return chi2;
 }
 
-int LMfit::autoiter () 
+void LMfit::autoiter() 
 {
-    wssr_before = (shake_before > 0. ? compute_wssr() : chi2);
+    wssr_before = (shake_before > 0. ? compute_wssr(a_orig, datsums) : chi2);
     fp prev_chi2 = chi2;
     verbose("\t === Levenberg-Marquardt method ===");
     info ("Initial values:  lambda=" + S(lambda) + "  WSSR=" + S(chi2));
     verbose ("Max. number of iterations: " + max_iterations);
+    fp stop_rel = getSettings()->get_f("lm-stop-rel-change");
+    fp max_lambda = getSettings()->get_f("lm-max-lambda");
     if (stop_rel > 0) {
         verbose ("Stopping when relative change of WSSR is "
-                  "second time below " + S (stop_rel * 100.) + "%");
+                  "twice in row below " + S (stop_rel * 100.) + "%");
     }
-    bool converged = false;
     int small_change_counter = 0;
     for (int iter = 0; !common_termination_criteria(iter); iter++) {
-        int result = do_iteration();
-        if (result < 0) {
-            warn ("Error when processing iteration " + S(iter+1) + ".");
-            return result;
-        }
-        if (result == 1) { //better fit
+        bool better_fit = do_iteration();
+        if (better_fit) { 
             fp d = prev_chi2 - chi2;
-            if (iter % output_one_of == 0)
-                info ("#" + S(iter_nr) + ":  WSSR=" + S(chi2) 
+            info ("#" + S(iter_nr) + ":  WSSR=" + S(chi2) 
                         + "  lambda=" + S(lambda) + "  d(WSSR)=" +  S(-d) 
                         + "  (" + S (d / prev_chi2 * 100) + "%)");  
-            if (d / prev_chi2 < stop_rel || chi2 == 0) { //another termination
-                small_change_counter++;                  // criterium:
-                if (small_change_counter >= 2 || chi2 == 0) { //second time
-                    info("Fit converged.");              // neglectable change 
-                    converged = true;                    // of chi2; or chi2==0
+            // another termination criterium: neglectable change of chi2
+            if (d / prev_chi2 < stop_rel || chi2 == 0) { 
+                small_change_counter++;
+                if (small_change_counter >= 2 || chi2 == 0) {
+                    info("Fit converged.");
                     break;
                 }
             }
@@ -92,22 +85,23 @@ int LMfit::autoiter ()
             prev_chi2 = chi2;
             iteration_plot(a);
         }
-        else { // result == 0, worse fit
-            info ("#" + S(iter_nr) + ": (WSSR=" + S(chi2_) 
-                    + ")  lambda=" + S(lambda));
+        else { // no better fit
+            info("#"+S(iter_nr)+": (WSSR="+S(chi2_)+")  lambda="+S(lambda));
+            if (lambda > max_lambda) { // another termination criterium
+                info("In L-M method: lambda=" + S(lambda) + " > " 
+                        + S(max_lambda) + ", stopped.");
+                break;
+            }
         }
     }
     post_fit (a, chi2);
-    return 1;
 }
 
-int LMfit::do_iteration()
+bool LMfit::do_iteration()
     //pre: init() callled
 {
-    if (na < 1) {
-        warn ("What am I to fit ?");
-        return -1;
-    }
+    if (na < 1) 
+        throw ExecuteError("No parameters to fit.");
     iter_nr++;
     alpha_ = alpha;
     for (int j = 0; j < na; j++) 
@@ -119,8 +113,8 @@ int LMfit::do_iteration()
 #endif /*debug*/
 
     // Matrix solution (Ax=b)  alpha_ * da == beta_
-    if (Jordan (alpha_, beta_, na) < 0)
-        return -1;
+    if (!Jordan (alpha_, beta_, na))
+        throw ExecuteError("Error when processing iteration " + S(iter_nr));
 
     // da is in beta_  
     if (getUI()->getVerbosity() >= 4) {
@@ -133,18 +127,18 @@ int LMfit::do_iteration()
         beta_[i] = a[i] + beta_[i];   // and now there is new a[] in beta_[] 
     verbose_lazy (print_matrix (beta_, 1, na, "Trying A"));
     //  compute chi2_
-    chi2_ = compute_wssr (beta_);
+    chi2_ = compute_wssr(beta_, datsums);
 
     if (chi2_ < chi2) { // better fitting
         chi2 = chi2_; 
         a = beta_;
-        compute_derivatives(a, alpha, beta);
-        lambda /= lambda_down_factor;
-        return 1;
+        compute_derivatives(a, datsums, alpha, beta);
+        lambda /= getSettings()->get_f("lm-lambda-down-factor");
+        return true;
     }
     else {// worse fitting
-        lambda *= lambda_up_factor;
-        return 0;
+        lambda *= getSettings()->get_f("lm-lambda-up-factor");
+        return false;
     }
 }    
 
