@@ -1,30 +1,23 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: LMfit.h,v 1.1.1.1 2003/06/17 20:07:53 wojdyr Exp $
+// $Id: LMfit.h,v 1.7 2006/07/10 15:58:50 wojdyr Exp $
 
-#ifndef LMfit__h__
-#define LMfit__h__
+#ifndef FITYK__LMFIT__H__
+#define FITYK__LMFIT__H__
 #include "common.h"
 #include <vector>
 #include <map>
 #include <string>
-#include "v_fit.h"
+#include "fit.h"
 
-/*     this class contains Levenberg-Marquardt method
- */
-
-
-class LMfit : public v_fit
+///           Levenberg-Marquardt method
+class LMfit : public Fit
 {
 public:
-    LMfit ();
-    ~LMfit ();
+    LMfit();
+    ~LMfit();
     fp init(); // called before do_iteration()/autoiter()
-    int autoiter ();
+    void autoiter();
 private:
-    fp lambda_starting_value;
-    fp lambda_up_factor;
-    fp lambda_down_factor;
-    fp stop_rel;
     fp shake_before;
     char shake_type;
     std::vector<fp> alpha, alpha_;            // matrices
@@ -33,15 +26,8 @@ private:
     fp chi2 , chi2_;
     fp lambda;
 
-    int do_iteration();
+    bool do_iteration();
 };
 
-inline void Swap (fp &fp1 ,fp &fp2) 
-{
-    fp swaptmp = fp1;
-    fp1 = fp2;
-    fp2 = swaptmp;
-}
-
 #endif
 
