@@ -1,5 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: NMfit.cpp,v 1.4 2005/05/13 18:54:48 wojdyr Exp $
+// $Id: NMfit.cpp,v 1.11 2006/07/10 15:58:50 wojdyr Exp $
 
 #include "common.h"
 #include "NMfit.h"
@@ -9,20 +9,19 @@
 #include <vector>
 #include <ui.h>
 
-#define TINY 1e-10
-
 using namespace std;
 
-NMfit::NMfit ()
-    :v_fit ('s', "Nelder-Mead simplex"), 
+NMfit::NMfit()
+    :Fit ("Nelder-Mead-simplex"), 
      min_rel_diff(0.0001), move_all(false), distrib_type('b'), move_mult(1.)
 {
-    default_max_iterations = 500;
+    /*
     fpar["min-fract-range"] = &min_rel_diff;
     bpar["move-all"] = &move_all;
     epar.insert(pair<string, Enum_string>("distrib-type", 
                                Enum_string (Distrib_enum, &distrib_type)));
     fpar["move-multiplier"] = &move_mult;
+    */
 }
 
 NMfit::~NMfit() {}
@@ -75,9 +74,9 @@ void NMfit::find_best_worst()
     }
 }
 
-int NMfit::autoiter ()
+void NMfit::autoiter()
 {
-    wssr_before = compute_wssr ();
+    wssr_before = compute_wssr(a_orig, datsums);
     info ("WSSR before starting simplex fit: " + S(wssr_before));
     for (int iter = 0; !termination_criteria (iter); iter ++) {
         iteration_plot(best->a);
@@ -86,7 +85,6 @@ int NMfit::autoiter ()
         find_best_worst();
     }
     post_fit (best->a, best->wssr);
-    return 0;
 }
 
 void NMfit::change_simplex()
@@ -144,8 +142,7 @@ void NMfit::compute_coord_sum()
 
 bool NMfit::termination_criteria (int iter)
 {
-    if (iter % output_one_of == 0) 
-        info ("#" + S(iter_nr) + " (ev:" + S(evaluations) + "): best:" 
+    info ("#" + S(iter_nr) + " (ev:" + S(evaluations) + "): best:" 
                 + S(best->wssr) + " worst:" + S(worst->wssr) + ", " 
                 + S(s_worst->wssr) + " [V * |" + S(volume_factor) + "|]");
     bool stop = false;
@@ -165,7 +162,7 @@ bool NMfit::termination_criteria (int iter)
     if (common_termination_criteria(iter))
         stop=true;
     fp r_diff = 2 * (worst->wssr - best->wssr) 
-                                        / (best->wssr + worst->wssr + TINY);
+                                        / (best->wssr + worst->wssr + EPSILON);
     if (r_diff < min_rel_diff) {
         info ("Relative difference between worst and best vertex is only "
                 + S(r_diff) + ". Stop");
@@ -177,7 +174,7 @@ bool NMfit::termination_criteria (int iter)
 void NMfit::compute_v (Vertex& v) 
 {
     assert (!v.a.empty()); 
-    v.wssr = compute_wssr(v.a); 
+    v.wssr = compute_wssr(v.a, datsums); 
     v.computed = true;
 }
 
