@@ -1,17 +1,14 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: NMfit.h,v 1.1.1.1 2003/06/17 20:07:53 wojdyr Exp $
+// $Id: NMfit.h,v 1.5 2006/07/10 15:58:50 wojdyr Exp $
 
-#ifndef NMfit__h__
-#define NMfit__h__
+#ifndef FITYK__NMFIT__H__
+#define FITYK__NMFIT__H__
 
 #include <vector>
 #include "common.h"
-#include "v_fit.h"
-
-/*     this class contains Nelder-Mead simplex method
- *     description of method: Numerical Recipes, www.nr.com
- */
+#include "fit.h"
 
+/// Vertex used in Nelder-Mead method 
 struct Vertex
 {
     std::vector<fp> a;
@@ -23,13 +20,14 @@ struct Vertex
     Vertex (std::vector<fp>& a_) : a(a_), computed(false) {}
 };
 
-class NMfit : public v_fit
+///              Nelder-Mead simplex method
+class NMfit : public Fit
 {
 public:
-    NMfit ();
-    ~NMfit ();
+    NMfit();
+    ~NMfit();
     fp init(); // called before autoiter()
-    int autoiter ();
+    void autoiter();
 private:
     int iteration;
     std::vector<Vertex> vertices;
