@@ -1,17 +1,12 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: data.cpp,v 1.19 2005/06/04 13:23:52 wojdyr Exp $
+// $Id: data.cpp,v 1.34 2006/09/12 00:28:45 wojdyr Exp $
 
 #include "common.h"
 #include "data.h" 
 #include "fileroutines.h"
 #include "ui.h"
 #include "numfuncs.h"
-
-//#include "datatrans.h" 
-// for faster compilation - don't include datatrans.h
-bool transform_data(std::string const& str, 
-                    std::vector<Point> const& old_points, 
-                    std::vector<Point> &new_points);
+#include "datatrans.h" 
 
 #include <math.h>
 #include <string.h>
@@ -42,26 +37,20 @@ string get_file_basename(const string &path)
 }
 
 
-Data *my_data;
-
-Data::Data ()
-    : d_was_changed(true) {}
-
-string Data::getInfo () const
+string Data::getInfo() const
 {
-    if (filename.empty()) 
-        return "No file loaded.";
-    else {
-        string s;
-        s = "Data: " + S(p.size()) + " points, " 
-            + S(active_p.size()) + " active.\n"
-            + "Filename: " + filename;
-        if (!title.empty())
-            s += "\nData title: " + title;
-        if (active_p.size() != p.size())
-            s += "\nActive data range: " + range_as_string();
-        return s;
-    }
+    string s;
+    if (p.empty())
+        s = "No data points.";
+    else
+        s = S(p.size()) + " points, " + S(active_p.size()) + " active.";
+    if (!filename.empty())
+        s += "\nFilename: " + filename;
+    if (!title.empty())
+        s += "\nData title: " + title;
+    if (active_p.size() != p.size())
+        s += "\nActive data range: " + range_as_string();
+    return s;
 }
 
 double Data::pdp11_f (char* fl)  //   function that converts:
@@ -78,38 +67,36 @@ double Data::pdp11_f (char* fl)  //   function that converts:
     return (znak==0 ? 1. : -1.) * h * pow(2., (double)unbiased);
 }
 
-char Data::guess_file_type (const string& filename)
+string Data::guess_file_type(string const& filename)
 { //using only filename extension
     if (filename.size() > 4) {
         string file_ext = filename.substr (filename.size() - 4, 4);
         if (file_ext == ".mca"   || file_ext == ".MCA") 
-            return 'm';
+            return "MCA";
         else if (file_ext == ".rit" || file_ext == ".RIT")
-            return 'r';
+            return "RIT";
         else if (file_ext == ".cpi" || file_ext == ".CPI")
-            return 'c';
+            return "CPI";
         else if (file_ext == ".raw" || file_ext == ".RAW")
-            return 's';
-        else 
-            return 'd';
+            return "BrukerRAW";
     }
-    else 
-        return 'd';
+    return "text";
 }
 
 void Data::clear()
 {
-    d_was_changed = true;
-    //gcc2.95 has no `basic_string<...>::clear ()'
-    filename = "";   //removing previos file
+    filename = "";   
     title = "";
+    given_type = "";
+    given_cols.clear();
     p.clear();
     active_p.clear();
-    col_nums.clear();
 }
 
 void Data::post_load()
 {
+    if (p.empty())
+        return;
     if (!p[0].sigma) {
         for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) 
             i->sigma = i->y > 1. ? sqrt (i->y) : 1.;
@@ -124,8 +111,14 @@ void Data::post_load()
 }
 
 void Data::recompute_y_bounds() {
-    y_min = y_max = p.front().y;
+    bool ini = false;
     for (vector<Point>::iterator i = p.begin(); i != p.end(); i++) {
+        if (!is_finite(i->y))
+            continue;
+        if (!ini) {
+            y_min = y_max = i->y;
+            ini = true;
+        }
         if (i->y < y_min)
             y_min = i->y;
         if (i->y > y_max)
@@ -140,7 +133,6 @@ int Data::load_arrays(const vector<fp> &x, const vector<fp> &y,
     assert(y.size() == size);
     assert(sigma.empty() || sigma.size() == size);
     clear();
-    filename = "-";
     title = data_title;
     if (sigma.empty()) 
         for (size_t i = 0; i < size; ++i)
@@ -154,55 +146,66 @@ int Data::load_arrays(const vector<fp> &x, const vector<fp> &y,
     return p.size();
 }
 
-
-int Data::load_file (const string &file, int type, const vector<int> &cols,
-                     bool append)
-{ 
-    if (type == 0) {                  // "detect" file format
-        type = guess_file_type(file);
+void Data::load_data_sum(vector<Data const*> const& dd)
+{
+    assert(!dd.empty());
+    clear();
+    title = dd[0]->get_title();
+    for (vector<Data const*>::const_iterator i = dd.begin()+1; 
+                                                          i != dd.end(); ++i)
+        title += " + " + (*i)->get_title();
+    for (vector<Data const*>::const_iterator i = dd.begin(); 
+                                                          i != dd.end(); ++i) {
+        vector<Point> const& points = (*i)->points();
+        p.insert(p.end(), points.begin(), points.end());
     }
+    sort(p.begin(), p.end());
+    x_step = find_step();
+    post_load();
+}
+
+void Data::load_file (string const& file, string const& type, 
+                     vector<int> const& cols, bool preview)
+{   
+    string const& ft = type.empty() ? guess_file_type(file) // "detect" format
+                                    : type;
     ifstream f (file.c_str(), ios::in | ios::binary);
     if (!f) {
-        warn ("Can't open file: " + file );
-        return -1;
-    }
-    if (append) {
-        filename = "";
-        col_nums.clear();
-    }
-    else {
-        clear(); //removing previous file
-        filename = file;   
-        col_nums = cols;
+        throw ExecuteError("Can't open file: " + file);
     }
+    clear(); //removing previous file
+    filename = file;   
+    given_type = type;
+    given_cols = cols;
 
-    if (type=='d')                            // x y x y ... 
+    if (ft == "text")                         // x y x y ... 
         load_xy_filetype(f, cols);
-    else if (type=='m')                       // .mca
+    else if (ft == "htext")                   // header\n x y x y ... 
+        load_header_xy_filetype(f, cols);
+    else if (ft == "MCA")                     // .mca
         load_mca_filetype(f);
-    else if (type=='r')                       // .rit
+    else if (ft == "RIT")                     // .rit
         load_rit_filetype(f);
-    else if (type=='c')                       // .cpi
+    else if (ft == "CPI")                     // .cpi
         load_cpi_filetype(f);
-    else if (type=='s')                       // .raw
+    else if (ft == "BrukerRAW")               // .raw
         load_siemensbruker_filetype(filename, this);
     else {                                  // other ?
-        warn ("Unknown filetype.");
-        return -1;
+        throw ExecuteError("Unknown filetype.");
+    }
+    if (preview) {
+        recompute_y_bounds();
+        return;
     }
     if (p.size() < 5)
         warn ("Only " + S(p.size()) + " data points found in file.");
-    if (!f.eof() && type != 'm') //!=mca; .mca doesn't have to reach EOF
+    if (!f.eof() && ft != "MCA") //!=mca; .mca doesn't have to reach EOF
         warn ("Unexpected char when reading " + S (p.size() + 1) + ". point");
-    if (p.empty())
-        return 0;
-
     post_load();
-    return p.size();
 }
 
 
-void Data::load_xy_filetype (ifstream& f, const vector<int>& columns)
+void Data::load_xy_filetype (ifstream& f, vector<int> const& columns)
 {
     /* format  x y \n x y \n x y \n ...
     *           38.834110      361
@@ -210,8 +213,10 @@ void Data::load_xy_filetype (ifstream& f, const vector<int>& columns)
     *           38.911500      352.431
     * delimiters: white spaces and  , : ;
      */
-    assert (columns.empty() || columns.size() == 2 || columns.size() == 3);
-    vector<int> cols = columns.empty() ? vector2<int>(1, 2) : columns;
+    if (columns.size() == 1 || columns.size() > 3)
+        throw ExecuteError("If columns are specified, two or three of them"
+                           " must be given (eg. 1,2,3)");
+    vector<int> const& cols = columns.empty() ? vector2<int>(1, 2) : columns;
     vector<fp> xy;
     int maxc = *max_element (cols.begin(), cols.end());
     int minc = *min_element (cols.begin(), cols.end());
@@ -264,6 +269,22 @@ void Data::load_xy_filetype (ifstream& f, const vector<int>& columns)
     x_step = find_step();
 }
 
+/// read first line as file's title, and than run load_xy_filetype()
+void Data::load_header_xy_filetype(ifstream& f, vector<int> const& columns)
+{
+    string s;
+    getline(f, s);
+    if (columns.size() > 1) {
+        int col = columns[1] - 1;
+        vector<string> v = split_string(s, " \t");
+        if (size(v) > col) 
+            title = v[col];
+    }
+    else
+        title = strip_string(s);
+    load_xy_filetype(f, columns);
+}
+
 void Data::load_mca_filetype (ifstream& f) 
 {
     typedef unsigned short int ui2b;
@@ -384,20 +405,12 @@ fp Data::get_y_at (fp x) const
     return y1 + (y2 - y1) * (x - x1) / (x2 - x1);
 }
 
-bool Data::transform(const string &s) 
+void Data::transform(const string &s) 
 {
-    vector<Point> new_p;
-    bool r = transform_data(s, p, new_p);
-    if (!r) {
-        warn("Syntax error.");
-        return false;
-    }
+    p = transform_data(s, p);
     //TODO history
-    p = new_p;
     sort(p.begin(), p.end());
     update_active_p();
-    d_was_changed = true;
-    return true;
 }
 
 void Data::update_active_p() 
@@ -405,9 +418,9 @@ void Data::update_active_p()
     // post: active_p sorted
 {
     active_p.clear();
-    for (unsigned int i = 0; i < p.size(); i++)
+    for (int i = 0; i < size(p); i++)
         if (p[i].is_active) 
-            active_p.push_back (i);
+            active_p.push_back(i);
 }
 
 // does anyone need it ? 
@@ -419,7 +432,6 @@ int Data::auto_range (fp y_level, fp x_margin)
         warn("No points loaded.");
         return -1;
     }
-    d_was_changed = true;
     bool state = (p.begin()->y >= y_level);
     for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) {
         if (state == (i->y >= y_level)) 
@@ -529,34 +541,45 @@ vector<Point>::const_iterator Data::get_point_at(fp x) const
     return lower_bound (p.begin(), p.end(), Point(x));
 }
 
-void Data::export_as_script (ostream& os)
-{
-    if (filename.empty()) {
-        os << "## no data loaded ##";
-        return;
-    }
-    //TODO optionally embed data (?)
-    os << "### data settings exported as script -- begin" << endl;
-    os << get_load_cmd();
-    //TODO transform history  
-    os << "### data settings -- end" << endl;
-}
-
-string Data::get_load_cmd()
-{
-    //TODO explicit filetype, when needed.
-    return "d.load " + join_vector(col_nums, ":") + " '" + filename + "'";
-}
-
-void Data::export_to_file (string filename, bool append) 
+void Data::export_to_file(string filename, vector<string> const& vt, 
+                          vector<string> const& ff_names,
+                          bool append) 
 {
     ofstream os(filename.c_str(), ios::out | (append ? ios::app : ios::trunc));
     if (!os) {
         warn ("Can't open file: " + filename);
         return;
     }
-    for (int i = 0; i < get_n(); i++)
-        os << get_x(i) << "\t" << get_y(i) << "\t" << get_sigma(i) << endl;
+    os << "# " << title << endl;
+    vector<string> cols;
+    if (vt.empty()) {
+        cols.push_back("x");
+        cols.push_back("y");
+        cols.push_back("s");
+    }
+    else {
+        for (vector<string>::const_iterator i=vt.begin(); i != vt.end(); ++i) 
+            if (startswith(*i, "*F")) {
+                for (vector<string>::const_iterator j = ff_names.begin(); 
+                                                    j != ff_names.end(); ++j)
+                    cols.push_back("%" + *j + string(*i, 2));
+            }
+            else
+                cols.push_back(*i);
+    }
+    os << join_vector(cols, "\t") << "\t#exported by fityk " VERSION << endl;
+
+    vector<vector<fp> > r;
+    bool only_active = !contains_element(vt, "a");
+    for (vector<string>::const_iterator i = cols.begin(); i != cols.end(); ++i) 
+        r.push_back(get_all_point_expressions(*i, this, only_active));
+
+    size_t nc = cols.size();
+    for (size_t i = 0; i != r[0].size(); ++i) {
+        for (size_t j = 0; j != nc; ++j) {
+            os << r[j][i] << (j < nc-1 ? '\t' : '\n'); 
+        }
+    }
 }
 
 
