@@ -1,8 +1,10 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: numfuncs.h,v 1.2 2005/05/21 16:35:44 wojdyr Exp $
+// $Id: numfuncs.h,v 1.6 2006/05/31 18:55:38 wojdyr Exp $
 
-#ifndef NUMFUNCS__H__
-#define NUMFUNCS__H__
+// it contains also declarations of functions from nmath.cpp
+
+#ifndef FITYK__NUMFUNCS__H__
+#define FITYK__NUMFUNCS__H__
 
 #include "common.h"
 
@@ -36,4 +38,17 @@ void prepare_spline_interpolation (std::vector<B_point> &bb);
 fp get_spline_interpolation(std::vector<B_point> &bb, fp x);
 fp get_linear_interpolation(std::vector<B_point> &bb, fp x);
 
-#endif //NUMFUNCS__H__
+//fp LnGammaE (fp x); /// log_e of Gamma function
+double digamma(double x);//in nmath.cpp
+double gammafn(double x);//in nmath.cpp
+double lgammafn(double x);//in nmath.cpp
+
+// random number utilities
+inline fp rand_1_1() { return 2.0 * rand() / RAND_MAX - 1.; }
+inline fp rand_0_1() { return static_cast<fp>(rand()) / RAND_MAX; }
+inline fp rand_uniform(fp a, fp b) { return a + rand_0_1() * (b-a); }
+inline bool rand_bool() { return rand() < RAND_MAX / 2; }
+fp rand_gauss();
+fp rand_cauchy();
+
+#endif 
