@@ -1,5 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: u_gnuplot.cpp,v 1.8 2005/05/21 16:35:45 wojdyr Exp $
+// $Id: u_gnuplot.cpp,v 1.14 2006/04/10 10:26:15 wojdyr Exp $
 
 // CLI-only file
 
@@ -15,15 +15,15 @@
 #include <errno.h>
 #include "data.h"
 #include "sum.h"
-#include "pcore.h"
-#include "other.h"
+#include "logic.h"
 #include "ui.h"
 
 using namespace std;
 
 char GnuPlot::path_to_gnuplot[]="gnuplot";
 
-GnuPlot::GnuPlot ()
+GnuPlot::GnuPlot()
+    : smooth_limit(0)
 {
     fork_and_make_pipe ();
 }
@@ -91,19 +91,25 @@ bool GnuPlot::gnuplot_pipe_ok()
     return true;
 }
 
-int GnuPlot::plot (const vector<fp>& workingA) 
+int GnuPlot::plot() 
 {
+    // plot only active data with sum
+    DataWithSum const* ds = AL->get_ds(AL->get_active_ds_position());
+    Data const* data = ds->get_data();
+    Sum const* sum = ds->get_sum();
     if (!gnuplot_pipe_ok())
         return -1;
-    my_sum->use_param_a_for_value(workingA);
-    string yfun = my_sum->sum_full_formula (workingA);
+    string yfun = ds->get_sum()->get_formula();
+    //gnuplot format is a bit different
+    replace_all(yfun, "^", "**");
+    replace_words(yfun, "ln", "log");
     very_verbose("Plotting function: " + yfun); 
     // Send commands through the pipe to gnuplot
-    int i_f = my_data->get_lower_bound_ac (my_core->view.left);
-    int i_l = my_data->get_upper_bound_ac (my_core->view.right);
+    int i_f = data->get_lower_bound_ac (AL->view.left);
+    int i_l = data->get_upper_bound_ac (AL->view.right);
     if (i_l - i_f > 0) { //plot data & sum
         bool function_as_points = (i_l - i_f > smooth_limit);
-        string plot_string = "plot "+ my_core->view.str() 
+        string plot_string = "plot "+ AL->view.str() 
             + " \'-\' title \"data\", ";
         if (function_as_points) {
             plot_string += " '-' title \"sum\" with line\n ";
@@ -114,22 +120,31 @@ int GnuPlot::plot (const vector<fp>& workingA)
             fprintf (gnuplot_pipe, plot_string.c_str());
         }
         if (fflush (gnuplot_pipe) != 0)
-            warn ("Flushing pipe program-to-gnuplot failed.");
-        for (int i = i_f; i < i_l; i++)
-            fprintf (gnuplot_pipe, 
-                     "%f  %f\n", my_data->get_x(i), my_data->get_y(i));
+            warn("Flushing pipe program-to-gnuplot failed.");
+        bool at_least_one_point = false;
+        for (int i = i_f; i < i_l; i++) {
+            fp x = data->get_x(i);
+            fp y = data->get_y(i);
+            if (is_finite(x) && is_finite(y)) {
+                fprintf(gnuplot_pipe, "%f  %f\n", x, y);
+                at_least_one_point = true;
+            }
+        }
+        if (!at_least_one_point)
+            fprintf(gnuplot_pipe, "0.0  0.0\n");
         fprintf (gnuplot_pipe, "e\n");//gnuplot needs 'e' at the end of data
         if (function_as_points) {
-            my_sum->use_param_a_for_value (workingA);
             for (int i = i_f; i < i_l; i++) {
-                fp x = my_data->get_x(i);
-                fprintf(gnuplot_pipe, "%f  %f\n", x, my_sum->value(x));
+                fp x = data->get_x(i);
+                fp y = sum->value(x);
+                if (is_finite(x) && is_finite(y))
+                    fprintf(gnuplot_pipe, "%f  %f\n", x, y);
             }
             fprintf(gnuplot_pipe, "e\n");
         }
     }
     else { // plot only sum
-        string plot_string =  "plot " + my_core->view.str() + " " + yfun 
+        string plot_string =  "plot " + AL->view.str() + " " + yfun 
             + " title \"function\"\n";
         fprintf (gnuplot_pipe, plot_string.c_str());
     }
