@@ -1,8 +1,8 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: u_gnuplot.h,v 1.4 2004/06/30 13:57:54 wojdyr Exp $
+// $Id: u_gnuplot.h,v 1.7 2005/12/05 18:43:07 wojdyr Exp $
 
-#ifndef gnuplot__H__
-#define gnuplot__H__
+#ifndef FITYK__GNUPLOT__H__
+#define FITYK__GNUPLOT__H__
 #include "common.h"
 #include <stdio.h>
 #include <vector>
@@ -12,11 +12,12 @@ class GnuPlot
 public:
     GnuPlot();
     ~GnuPlot();
-    int plot (const std::vector<fp>& workingA);
+    int plot();
     void raw_command(char *command);// no syntax checking
     static char path_to_gnuplot[] ; 
 
 private:
+    int smooth_limit; 
     FILE *gnuplot_pipe;
 
     void fork_and_make_pipe ();
